function fluofitplot(beta, opt, desc, fid, ptype)
%FLUOFITPLOT Plots fitted fluorescence spectra.
%   FLUOFITPLOT(BETA, OPT, DESC) plots the fitted fluorescence spectra in
%   BETA, DESC (with fitting options OPT), and prints the fitting
%   parameters.
%   FLUOFITPLOT(..., FID) writes the parameters to FID instead of stdout.
%   FLUOFITPLOT(..., PTYPE) plots only part.
%      PTYPE = -1 plots the virtual excitation spectrum, while PTYPE > 0
%      selects a replicate.
%
%   EXAMPLE:
%      fluofitplot(beta, opt, fluofitdesc(data, em, ex, opt));
%
%   AUTHOR:
%      Wouter Caarls, MPIbpc <wouter@caarls.org>

    if nargin < 4
        fid = 1;
    end
    
    if nargin < 5
        ptype = 0;
    end

    desc.exs = length(desc.ex);
    desc.ems = length(desc.em);

    colors = 'rgbcmy';
    styles = {[0.25 0.5 0.25 0.5], [1 0.5 1 0.5], [0.25 0.5 1 0.5]};
    markers = 'osd';
    
    lw = 0.5;
    
    if opt.wavenumber
        em = 10^7./desc.em;
        ex = 10^7./desc.ex;
    else
        em = desc.em;
        ex = desc.ex;
    end
    
    if desc.reps > 1
        pr = desc.reps;
        pc = desc.exs+1;
    else
        pc = max(2, ceil(sqrt(desc.exs)));
        pr = ceil(desc.exs/pc);
    end

    hs = opt.bands*3;
    or = (length(beta)-hs)/desc.reps;
    
    fprintf(fid, 'Band   rho    FWHM   pos\n');
    for b = 1:opt.bands
        pos = beta((b-1)*3+3);
        if opt.wavenumber
            pos = 10^7/pos;
        end
        fprintf(fid, '%-6d %-6.2f %-6.0f %-6.2f\n', b, beta((b-1)*3+1), beta((b-1)*3+2), pos);
    end

    sse = 0;
    ssm = 0;
    area = zeros(1, opt.bands);
    
    for r = 1:desc.reps
        switch ptype
            case -1
                m = max(max(max(desc.data(:,:,:))));
            case 0
                m = max(max(desc.data(:,:,r)));
            otherwise
                m = max(max(desc.data(:,ptype,:)));
        end
        
        fprintf(fid, '\nBand ');
        fprintf(fid, '%6.0f ', ex);
        fprintf(fid, '\n');
    
        for b = 1:opt.bands
            fprintf(fid, '%-4d ', b);
            for i = 1:length(desc.ex)
                fprintf(fid, '%6.2f ', beta(hs+(r-1)*or+(b-1)*desc.exs+i));
            end
            fprintf(fid, '\n');
        end
        
        for i = 1:(desc.exs+1)
            if ptype == -1 || i == ptype
                h = subplot(1, desc.reps, r);
                %pos = get(gca, 'Position');
                %set(gca, 'Position', [pos(1:2) pos(3) pos(3)/desc.reps]);
            elseif ptype == 0
                if desc.reps > 1
                    h = subplot(pr, pc, (r-1)*(desc.exs+1) + i, 'align');
                else
                    h = subplot(pr, pc, i);
                end
                pos = get(gca, 'Position');
                set(gca, 'Position', [pos(1:2) pos(3) pos(3)/pr*pc]);
            end 
            hold on
            plotted = 0;
            
            if i <= desc.exs && (ptype == 0 || i == ptype)
                plotted = 1;
                plot(em, desc.data(:, i, r), 'k.', 'markersize', 4);%, 'LineWidth', lw);
                axis([floor(min(em)/10)*10 ceil(max(em)/10)*10+0.0001 -0.2 m]);

                te = zeros(1, desc.ems);
                for b = 1:opt.bands
                    be = alogn(beta((b-1)*3+1), beta((b-1)*3+2), beta((b-1)*3+3), beta(hs+(r-1)*or+(b-1)*desc.exs+i), desc.em);
                    te = te + be;
                    
                    style = styles{b};
                    
                    dashline(em, be, style(1), style(2), style(3), style(4), colors(b));%, 'LineWidth', lw);
                    area(b) = area(b) + curvearea(desc.em, be); 
                end

                plot(em, te, 'k-');%, 'LineWidth', lw);
                plot(em, desc.data(:, i, r)-te', 'k-');%, 'LineWidth', lw);

                y = desc.data(:, i, r);
                nz = y>0;
                
                sse = sse + sum((y(nz)'-te(nz)).^2);
                ssm = ssm + sum((y(nz)'-mean(y(nz)')).^2);
            elseif i > desc.exs && ptype <= 0
                plotted = 1;
                axis([floor(min(ex)/10)*10 ceil(max(ex)/10)*10+0.0001 0 m]);
                % Plot virtual excitation spectra
                fprintf(fid, 'Band   rho    FWHM   pos    height\n');
                for b = 1:opt.bands
                    y = beta(hs+(r-1)*or+(b-1)*desc.exs+[1:desc.exs]);
                    if opt.wavenumber
                        beta0 = [1.5 3000 10^7/420 max(y)];
                    else
                        beta0 = [1.5 60 420 max(y)];
                    end

                    p = sum(desc.data(:,:,r), 1)>0;
                                        
                    ebeta = nlinfit(desc.ex(p), y(p), @alognmdl, beta0, statset('DerivStep', 0.01));
                    
                    iex = min(desc.ex):max(desc.ex);
                    if opt.wavenumber
                        iiex = 10^7./iex;
                    else
                        iiex = ex;
                    end
                    
                    plot(ex, y, [colors(b) markers(b)], iiex, alognmdl(ebeta, iex), [colors(b) '-'], 'MarkerSize', 3);%, 'LineWidth', lw);

                    if opt.wavenumber
                        pos = 10^7/ebeta(3);
                    else
                        pos = ebeta(3);
                    end
                    
                    fprintf(fid, '%-6d %-6.2f %-6.0f %-6.2f %-6.2f\n', b, ebeta(1), ebeta(2), pos, ebeta(4));
                end
            end

            if plotted
                set(h, 'YTick', 0:tickspacing(m, 4):m);
                set(h, 'XGrid', 'on', 'YGrid', 'on');

                if desc.reps > 1
                    if ptype == 0
                        ax=get(h,'Position');
                        sz = ax([3 4]).*[1.2 1.2];
                        set(h, 'Position', [ax([1 2]) sz]);
                    end

                    if i <= desc.exs
                        set(h, 'XTick', (ceil(min(em)/50)*50):100:(floor(max(em)/50)*50));
                    else
                        set(h, 'XTick', (ceil(min(ex)/50)*50):50:(floor(max(ex)/50)*50));
                    end

                    if ptype == 0
                        if i > 1
                            if i <= desc.exs
                                set(h, 'YTickLabel', []);
                            else
                                set(h, 'YAxisLocation', 'right');
                            end
                        end
                        if r < desc.reps
                            set(h, 'XTickLabel', []);
                            if r == 1 && i <= desc.exs
                                title([num2str(ex(i)) 'nm']);
                            end
                        end
                    elseif ptype == -1
                        title('Virtual excitation');
                        %title(['EtAc + ' num2str(r-1) '% H_2O']);
                        xlabel('Excitation (nm)');
                        ylabel('Intensity (a.u.)');
                    else
                        %title(['Excitation at ' num2str(ex(ptype)) 'nm']);
                        title(['EtAc + ' num2str(r-1) '% H_2O']);
                        xlabel('Emission (nm)');
                        ylabel('Intensity (a.u.)');
                    end
                else
                    if i <= desc.exs
                        title([num2str(ex(i)) 'nm']);
                        set(h, 'XTick', (ceil(min(em)/50)*50):100:(floor(max(em)/50)*50));
                    else
                        title('Virtual excitation');
                        set(h, 'XTick', (ceil(min(ex)/50)*50):50:(floor(max(ex)/50)*50));
                    end
                end

                hold off
            end
        end
    end
    
    fprintf(fid, 'Band   area\n');
    for b = 1:opt.bands
        fprintf(fid, '%-6d %f\n', b, abs(area(b)));
    end

    fprintf(fid, '\nsse = %f\n', sse);
    fprintf(fid, 'rsq = %f\n\n', 1-sse/ssm);
end

function y = tickspacing(x, t)

    spc = [2 4 5 10];
    s = spc.^(round(log(x/t)./log(spc)));
    [d, i] = min(abs(s - x/t));
    y = s(i);

end



